@@ -21,14 +21,14 @@
 
 """astroid packaging information"""
 
-version = "2.4.1"
+version = "2.4.2"
 numversion = tuple(int(elem) for elem in version.split(".") if elem.isdigit())
 
 extras_require = {}
 install_requires = [
-    "lazy_object_proxy==1.4.*",
+    "lazy_object_proxy",
     "six",
-    "wrapt~=1.11",
+    "wrapt",
     'typed-ast>=1.4.0,<1.5;implementation_name== "cpython" and python_version<"3.8"',
 ]
 
