@@ -1,8 +1,14 @@
-# Copyright (c) 2014-2018 Claudiu Popa <pcmanticore@gmail.com>
+# -*- coding: utf-8 -*-
+# Copyright (c) 2014-2020 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2014-2015 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
 # Copyright (c) 2015-2016 Ceridwen <ceridwenv@gmail.com>
 # Copyright (c) 2015 Rene Zhang <rz99@cornell.edu>
 # Copyright (c) 2018 Bryce Guinta <bryce.paul.guinta@gmail.com>
+# Copyright (c) 2018 Ville Skyttä <ville.skytta@iki.fi>
+# Copyright (c) 2019 Stanislav Levin <slev@altlinux.org>
+# Copyright (c) 2019 David Liu <david@cs.toronto.edu>
+# Copyright (c) 2019 Bryce Guinta <bryce.guinta@protonmail.com>
+# Copyright (c) 2019 Frédéric Chapoton <fchapoton2@gmail.com>
 
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
 # For details: https://github.com/PyCQA/astroid/blob/master/COPYING.LESSER
@@ -15,7 +21,6 @@ from textwrap import dedent
 import six
 from astroid import (
     MANAGER,
-    Instance,
     UseInferenceDefault,
     AttributeInferenceError,
     inference_tip,
@@ -148,14 +153,14 @@ def register_builtin_transform(transform, builtin_name):
     )
 
 
-def _generic_inference(node, context, node_type, transform):
+def _container_generic_inference(node, context, node_type, transform):
     args = node.args
     if not args:
         return node_type()
     if len(node.args) > 1:
         raise UseInferenceDefault()
 
-    arg, = args
+    (arg,) = args
     transformed = transform(arg)
     if not transformed:
         try:
@@ -170,14 +175,24 @@ def _generic_inference(node, context, node_type, transform):
     return transformed
 
 
-def _generic_transform(arg, klass, iterables, build_elts):
+def _container_generic_transform(arg, context, klass, iterables, build_elts):
     if isinstance(arg, klass):
         return arg
     elif isinstance(arg, iterables):
-        if not all(isinstance(elt, nodes.Const) for elt in arg.elts):
-            raise UseInferenceDefault()
-        elts = [elt.value for elt in arg.elts]
+        if all(isinstance(elt, nodes.Const) for elt in arg.elts):
+            elts = [elt.value for elt in arg.elts]
+        else:
+            # TODO: Does not handle deduplication for sets.
+            elts = []
+            for element in arg.elts:
+                inferred = helpers.safe_infer(element, context=context)
+                if inferred:
+                    evaluated_object = nodes.EvaluatedObject(
+                        original=element, value=inferred
+                    )
+                    elts.append(evaluated_object)
     elif isinstance(arg, nodes.Dict):
+        # Dicts need to have consts as strings already.
         if not all(isinstance(elt[0], nodes.Const) for elt in arg.items):
             raise UseInferenceDefault()
         elts = [item[0].value for item in arg.items]
@@ -187,20 +202,26 @@ def _generic_transform(arg, klass, iterables, build_elts):
         elts = arg.value
     else:
         return
-    return klass.from_constants(elts=build_elts(elts))
+    return klass.from_elements(elts=build_elts(elts))
 
 
-def _infer_builtin(node, context, klass=None, iterables=None, build_elts=None):
+def _infer_builtin_container(
+    node, context, klass=None, iterables=None, build_elts=None
+):
     transform_func = partial(
-        _generic_transform, klass=klass, iterables=iterables, build_elts=build_elts
+        _container_generic_transform,
+        context=context,
+        klass=klass,
+        iterables=iterables,
+        build_elts=build_elts,
     )
 
-    return _generic_inference(node, context, klass, transform_func)
+    return _container_generic_inference(node, context, klass, transform_func)
 
 
 # pylint: disable=invalid-name
 infer_tuple = partial(
-    _infer_builtin,
+    _infer_builtin_container,
     klass=nodes.Tuple,
     iterables=(
         nodes.List,
@@ -214,7 +235,7 @@ infer_tuple = partial(
 )
 
 infer_list = partial(
-    _infer_builtin,
+    _infer_builtin_container,
     klass=nodes.List,
     iterables=(
         nodes.Tuple,
@@ -228,14 +249,14 @@ infer_list = partial(
 )
 
 infer_set = partial(
-    _infer_builtin,
+    _infer_builtin_container,
     klass=nodes.Set,
     iterables=(nodes.List, nodes.Tuple, objects.FrozenSet, objects.DictKeys),
     build_elts=set,
 )
 
 infer_frozenset = partial(
-    _infer_builtin,
+    _infer_builtin_container,
     klass=objects.FrozenSet,
     iterables=(nodes.List, nodes.Tuple, nodes.Set, objects.FrozenSet, objects.DictKeys),
     build_elts=frozenset,
@@ -283,7 +304,7 @@ def infer_dict(node, context=None):
 
     If a case can't be inferred, we'll fallback to default inference.
     """
-    call = arguments.CallSite.from_call(node)
+    call = arguments.CallSite.from_call(node, context=context)
     if call.has_invalid_arguments() or call.has_invalid_keywords():
         raise UseInferenceDefault
 
@@ -471,6 +492,35 @@ def infer_callable(node, context=None):
     return nodes.Const(inferred.callable())
 
 
+def infer_property(node, context=None):
+    """Understand `property` class
+
+    This only infers the output of `property`
+    call, not the arguments themselves.
+    """
+    if len(node.args) < 1:
+        # Invalid property call.
+        raise UseInferenceDefault
+
+    getter = node.args[0]
+    try:
+        inferred = next(getter.infer(context=context))
+    except InferenceError:
+        raise UseInferenceDefault
+
+    if not isinstance(inferred, (nodes.FunctionDef, nodes.Lambda)):
+        raise UseInferenceDefault
+
+    return objects.Property(
+        function=inferred,
+        name=inferred.name,
+        doc=getattr(inferred, "doc", None),
+        lineno=node.lineno,
+        parent=node,
+        col_offset=node.col_offset,
+    )
+
+
 def infer_bool(node, context=None):
     """Understand bool calls."""
     if len(node.args) > 1:
@@ -488,7 +538,7 @@ def infer_bool(node, context=None):
     if inferred is util.Uninferable:
         return util.Uninferable
 
-    bool_value = inferred.bool_value()
+    bool_value = inferred.bool_value(context=context)
     if bool_value is util.Uninferable:
         return util.Uninferable
     return nodes.Const(bool_value)
@@ -558,7 +608,7 @@ def infer_issubclass(callnode, context=None):
     :rtype nodes.Const: Boolean Const value of the `issubclass` call
     :raises UseInferenceDefault: If the node cannot be inferred
     """
-    call = arguments.CallSite.from_call(callnode)
+    call = arguments.CallSite.from_call(callnode, context=context)
     if call.keyword_arguments:
         # issubclass doesn't support keyword arguments
         raise UseInferenceDefault("TypeError: issubclass() takes no keyword arguments")
@@ -605,7 +655,7 @@ def infer_isinstance(callnode, context=None):
 
     :raises UseInferenceDefault: If the node cannot be inferred
     """
-    call = arguments.CallSite.from_call(callnode)
+    call = arguments.CallSite.from_call(callnode, context=context)
     if call.keyword_arguments:
         # isinstance doesn't support keyword arguments
         raise UseInferenceDefault("TypeError: isinstance() takes no keyword arguments")
@@ -662,7 +712,7 @@ def infer_len(node, context=None):
     :param context.InferenceContext: node context
     :rtype nodes.Const: a Const node with the inferred length, if possible
     """
-    call = arguments.CallSite.from_call(node)
+    call = arguments.CallSite.from_call(node, context=context)
     if call.keyword_arguments:
         raise UseInferenceDefault("TypeError: len() must take no keyword arguments")
     if len(call.positional_arguments) != 1:
@@ -672,7 +722,7 @@ def infer_len(node, context=None):
         )
     [argument_node] = call.positional_arguments
     try:
-        return nodes.Const(helpers.object_len(argument_node))
+        return nodes.Const(helpers.object_len(argument_node, context=context))
     except (AstroidTypeError, InferenceError) as exc:
         raise UseInferenceDefault(str(exc)) from exc
 
@@ -684,7 +734,7 @@ def infer_str(node, context=None):
     :param context.InferenceContext: node context
     :rtype nodes.Const: a Const containing an empty string
     """
-    call = arguments.CallSite.from_call(node)
+    call = arguments.CallSite.from_call(node, context=context)
     if call.keyword_arguments:
         raise UseInferenceDefault("TypeError: str() must take no keyword arguments")
     try:
@@ -700,14 +750,14 @@ def infer_int(node, context=None):
     :param context.InferenceContext: node context
     :rtype nodes.Const: a Const containing the integer value of the int() call
     """
-    call = arguments.CallSite.from_call(node)
+    call = arguments.CallSite.from_call(node, context=context)
     if call.keyword_arguments:
         raise UseInferenceDefault("TypeError: int() must take no keyword arguments")
 
     if call.positional_arguments:
         try:
             first_value = next(call.positional_arguments[0].infer(context=context))
-        except InferenceError as exc:
+        except (InferenceError, StopIteration) as exc:
             raise UseInferenceDefault(str(exc)) from exc
 
         if first_value is util.Uninferable:
@@ -743,7 +793,7 @@ def infer_dict_fromkeys(node, context=None):
         new_node.postinit(elements)
         return new_node
 
-    call = arguments.CallSite.from_call(node)
+    call = arguments.CallSite.from_call(node, context=context)
     if call.keyword_arguments:
         raise UseInferenceDefault("TypeError: int() must take no keyword arguments")
     if len(call.positional_arguments) not in {1, 2}:
@@ -797,6 +847,7 @@ def infer_dict_fromkeys(node, context=None):
 register_builtin_transform(infer_bool, "bool")
 register_builtin_transform(infer_super, "super")
 register_builtin_transform(infer_callable, "callable")
+register_builtin_transform(infer_property, "property")
 register_builtin_transform(infer_getattr, "getattr")
 register_builtin_transform(infer_hasattr, "hasattr")
 register_builtin_transform(infer_tuple, "tuple")
