@@ -3,6 +3,7 @@
 # Copyright (c) 2016-2017 Łukasz Rogalski <rogalski.91@gmail.com>
 # Copyright (c) 2017 Derek Gustafson <degustaf@gmail.com>
 # Copyright (c) 2018 Ioana Tagirta <ioana.tagirta@gmail.com>
+# Copyright (c) 2019 Hugo van Kemenade <hugovk@users.noreply.github.com>
 
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
 # For details: https://github.com/PyCQA/astroid/blob/master/COPYING.LESSER
@@ -10,9 +11,6 @@ import sys
 
 import astroid
 
-PY34 = sys.version_info >= (3, 4)
-PY35 = sys.version_info >= (3, 5)
-
 
 def _collections_transform():
     return astroid.parse(
@@ -54,9 +52,7 @@ def _deque_mock():
         def __nonzero__(self): return bool(self.iterable)
         def __contains__(self, o): return o in self.iterable
         def __len__(self): return len(self.iterable)
-        def __copy__(self): return deque(self.iterable)"""
-    if PY35:
-        base_deque_class += """
+        def __copy__(self): return deque(self.iterable)
         def copy(self): return deque(self.iterable)
         def index(self, x, start=0, end=0): return 0
         def insert(self, x, i): pass
@@ -72,9 +68,6 @@ def _ordered_dict_mock():
     base_ordered_dict_class = """
     class OrderedDict(dict):
         def __reversed__(self): return self[::-1]
-    """
-    if PY34:
-        base_ordered_dict_class += """
         def move_to_end(self, key, last=False): pass"""
     return base_ordered_dict_class
 
