@@ -2,7 +2,7 @@
 # Copyright (c) 2006-2011, 2013-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
 # Copyright (c) 2012 FELD Boris <lothiraldan@gmail.com>
 # Copyright (c) 2013-2014 Google, Inc.
-# Copyright (c) 2014-2018 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2014-2020 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2014 Eevee (Alex Munroe) <amunroe@yelp.com>
 # Copyright (c) 2015-2016 Ceridwen <ceridwenv@gmail.com>
 # Copyright (c) 2015 Dmitry Pribysh <dmand@yandex.ru>
@@ -10,10 +10,13 @@
 # Copyright (c) 2017 Michał Masłowski <m.maslowski@clearcode.cc>
 # Copyright (c) 2017 Calen Pennington <cale@edx.org>
 # Copyright (c) 2017 Łukasz Rogalski <rogalski.91@gmail.com>
+# Copyright (c) 2018-2019 Nick Drozd <nicholasdrozd@gmail.com>
+# Copyright (c) 2018 Daniel Martin <daniel.martin@crowdstrike.com>
+# Copyright (c) 2018 Ville Skyttä <ville.skytta@iki.fi>
 # Copyright (c) 2018 Bryce Guinta <bryce.paul.guinta@gmail.com>
-# Copyright (c) 2018 Nick Drozd <nicholasdrozd@gmail.com>
 # Copyright (c) 2018 Ashley Whetter <ashley@awhetter.co.uk>
 # Copyright (c) 2018 HoverHell <hoverhell@gmail.com>
+# Copyright (c) 2020 Leandro T. C. Melo <ltcmelo@gmail.com>
 
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
 # For details: https://github.com/PyCQA/astroid/blob/master/COPYING.LESSER
@@ -25,6 +28,7 @@ import functools
 import itertools
 import operator
 
+import wrapt
 from astroid import bases
 from astroid import context as contextmod
 from astroid import exceptions
@@ -38,22 +42,24 @@ from astroid import util
 
 
 MANAGER = manager.AstroidManager()
+# Prevents circular imports
+objects = util.lazy_import("objects")
 
 
 # .infer method ###############################################################
 
 
 def infer_end(self, context=None):
-    """inference's end for node such as Module, ClassDef, FunctionDef,
-    Const...
+    """Inference's end for nodes that yield themselves on inference
 
+    These are objects for which inference does not have any semantic,
+    such as Module or Consts.
     """
     yield self
 
 
 nodes.Module._infer = infer_end
 nodes.ClassDef._infer = infer_end
-nodes.FunctionDef._infer = infer_end
 nodes.Lambda._infer = infer_end
 nodes.Const._infer = infer_end
 nodes.Slice._infer = infer_end
@@ -71,17 +77,22 @@ def _infer_sequence_helper(node, context=None):
             if not hasattr(starred, "elts"):
                 raise exceptions.InferenceError(node=node, context=context)
             values.extend(_infer_sequence_helper(starred))
+        elif isinstance(elt, nodes.NamedExpr):
+            value = helpers.safe_infer(elt.value, context)
+            if not value:
+                raise exceptions.InferenceError(node=node, context=context)
+            values.append(value)
         else:
             values.append(elt)
     return values
 
 
 @decorators.raise_if_nothing_inferred
-@decorators.path_wrapper
 def infer_sequence(self, context=None):
-    if not any(isinstance(e, nodes.Starred) for e in self.elts):
-        yield self
-    else:
+    has_starred_named_expr = any(
+        isinstance(e, (nodes.Starred, nodes.NamedExpr)) for e in self.elts
+    )
+    if has_starred_named_expr:
         values = _infer_sequence_helper(self, context)
         new_seq = type(self)(
             lineno=self.lineno, col_offset=self.col_offset, parent=self.parent
@@ -89,6 +100,8 @@ def infer_sequence(self, context=None):
         new_seq.postinit(values)
 
         yield new_seq
+    else:
+        yield self
 
 
 nodes.List._infer = infer_sequence
@@ -246,15 +259,6 @@ def infer_import(self, context=None, asname=True):
 nodes.Import._infer = infer_import
 
 
-def infer_name_module(self, name):
-    context = contextmod.InferenceContext()
-    context.lookupname = name
-    return self.infer(context, asname=False)
-
-
-nodes.Import.infer_name_module = infer_name_module
-
-
 @decorators.raise_if_nothing_inferred
 @decorators.path_wrapper
 def infer_import_from(self, context=None, asname=True):
@@ -284,7 +288,6 @@ def infer_import_from(self, context=None, asname=True):
 nodes.ImportFrom._infer = infer_import_from
 
 
-@decorators.raise_if_nothing_inferred
 def infer_attribute(self, context=None):
     """infer an Attribute node by using getattr on the associated object"""
     for owner in self.expr.infer(context):
@@ -308,21 +311,28 @@ def infer_attribute(self, context=None):
                 except exceptions._NonDeducibleTypeHierarchy:
                     # Can't determine anything useful.
                     pass
+        elif not context:
+            context = contextmod.InferenceContext()
 
         try:
             context.boundnode = owner
             yield from owner.igetattr(self.attrname, context)
-            context.boundnode = None
-        except (exceptions.AttributeInferenceError, exceptions.InferenceError):
-            context.boundnode = None
-        except AttributeError:
-            # XXX method / function
+        except (
+            exceptions.AttributeInferenceError,
+            exceptions.InferenceError,
+            AttributeError,
+        ):
+            pass
+        finally:
             context.boundnode = None
     return dict(node=self, context=context)
 
 
-nodes.Attribute._infer = decorators.path_wrapper(infer_attribute)
-nodes.AssignAttr.infer_lhs = infer_attribute  # # won't work with a path wrapper
+nodes.Attribute._infer = decorators.raise_if_nothing_inferred(
+    decorators.path_wrapper(infer_attribute)
+)
+# won't work with a path wrapper
+nodes.AssignAttr.infer_lhs = decorators.raise_if_nothing_inferred(infer_attribute)
 
 
 @decorators.raise_if_nothing_inferred
@@ -344,7 +354,6 @@ nodes.Global._infer = infer_global
 _SUBSCRIPT_SENTINEL = object()
 
 
-@decorators.raise_if_nothing_inferred
 def infer_subscript(self, context=None):
     """Inference for subscripts
 
@@ -368,13 +377,13 @@ def infer_subscript(self, context=None):
             index_value = _SUBSCRIPT_SENTINEL
             if value.__class__ == bases.Instance:
                 index_value = index
+            elif index.__class__ == bases.Instance:
+                instance_as_index = helpers.class_instance_as_index(index)
+                if instance_as_index:
+                    index_value = instance_as_index
             else:
-                if index.__class__ == bases.Instance:
-                    instance_as_index = helpers.class_instance_as_index(index)
-                    if instance_as_index:
-                        index_value = instance_as_index
-                else:
-                    index_value = index
+                index_value = index
+
             if index_value is _SUBSCRIPT_SENTINEL:
                 raise exceptions.InferenceError(node=self, context=context)
 
@@ -401,8 +410,10 @@ def infer_subscript(self, context=None):
     return None
 
 
-nodes.Subscript._infer = decorators.path_wrapper(infer_subscript)
-nodes.Subscript.infer_lhs = infer_subscript
+nodes.Subscript._infer = decorators.raise_if_nothing_inferred(
+    decorators.path_wrapper(infer_subscript)
+)
+nodes.Subscript.infer_lhs = decorators.raise_if_nothing_inferred(infer_subscript)
 
 
 @decorators.raise_if_nothing_inferred
@@ -817,7 +828,6 @@ nodes.AugAssign._infer = infer_augassign
 
 
 @decorators.raise_if_nothing_inferred
-@decorators.path_wrapper
 def infer_arguments(self, context=None):
     name = context.lookupname
     if name is None:
@@ -834,9 +844,8 @@ def infer_assign(self, context=None):
     """infer a AssignName/AssignAttr: need to inspect the RHS part of the
     assign node
     """
-    stmt = self.statement()
-    if isinstance(stmt, nodes.AugAssign):
-        return stmt.infer(context)
+    if isinstance(self.parent, nodes.AugAssign):
+        return self.parent.infer(context)
 
     stmts = list(self.assigned_stmts(context=context))
     return bases._infer_stmts(stmts, context)
@@ -862,7 +871,6 @@ nodes.EmptyNode._infer = infer_empty_node
 
 
 @decorators.raise_if_nothing_inferred
-@decorators.path_wrapper
 def infer_index(self, context=None):
     return self.value.infer(context)
 
@@ -886,15 +894,7 @@ def instance_getitem(self, index, context=None):
             "Could not find __getitem__ for {node!r}.", node=self, context=context
         )
 
-    try:
-        return next(method.infer_call_result(self, new_context))
-    except StopIteration as exc:
-        raise exceptions.InferenceError(
-            message="Inference for {node!r}[{index!s}] failed.",
-            node=self,
-            index=index,
-            context=context,
-        ) from exc
+    return next(method.infer_call_result(self, new_context))
 
 
 bases.Instance.getitem = instance_getitem
@@ -915,3 +915,80 @@ def _populate_context_lookup(call, context):
     for keyword in keywords:
         context_lookup[keyword.value] = context
     return context_lookup
+
+
+@decorators.raise_if_nothing_inferred
+def infer_ifexp(self, context=None):
+    """Support IfExp inference
+
+    If we can't infer the truthiness of the condition, we default
+    to inferring both branches. Otherwise, we infer either branch
+    depending on the condition.
+    """
+    both_branches = False
+    # We use two separate contexts for evaluating lhs and rhs because
+    # evaluating lhs may leave some undesired entries in context.path
+    # which may not let us infer right value of rhs.
+
+    context = context or contextmod.InferenceContext()
+    lhs_context = contextmod.copy_context(context)
+    rhs_context = contextmod.copy_context(context)
+    try:
+        test = next(self.test.infer(context=context.clone()))
+    except exceptions.InferenceError:
+        both_branches = True
+    else:
+        if test is not util.Uninferable:
+            if test.bool_value():
+                yield from self.body.infer(context=lhs_context)
+            else:
+                yield from self.orelse.infer(context=rhs_context)
+        else:
+            both_branches = True
+    if both_branches:
+        yield from self.body.infer(context=lhs_context)
+        yield from self.orelse.infer(context=rhs_context)
+
+
+nodes.IfExp._infer = infer_ifexp
+
+
+# pylint: disable=dangerous-default-value
+@wrapt.decorator
+def _cached_generator(func, instance, args, kwargs, _cache={}):
+    node = args[0]
+    try:
+        return iter(_cache[func, id(node)])
+    except KeyError:
+        result = func(*args, **kwargs)
+        # Need to keep an iterator around
+        original, copy = itertools.tee(result)
+        _cache[func, id(node)] = list(copy)
+        return original
+
+
+# When inferring a property, we instantiate a new `objects.Property` object,
+# which in turn, because it inherits from `FunctionDef`, sets itself in the locals
+# of the wrapping frame. This means that everytime we infer a property, the locals
+# are mutated with a new instance of the property. This is why we cache the result
+# of the function's inference.
+@_cached_generator
+def infer_functiondef(self, context=None):
+    if not self.decorators or not bases._is_property(self):
+        yield self
+        return dict(node=self, context=context)
+
+    prop_func = objects.Property(
+        function=self,
+        name=self.name,
+        doc=self.doc,
+        lineno=self.lineno,
+        parent=self.parent,
+        col_offset=self.col_offset,
+    )
+    prop_func.postinit(body=[], args=self.args)
+    yield prop_func
+    return dict(node=self, context=context)
+
+
+nodes.FunctionDef._infer = infer_functiondef
