@@ -1,9 +1,21 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2014-2016 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2014-2018, 2020 Claudiu Popa <pcmanticore@gmail.com>
 # Copyright (c) 2014 Google, Inc.
+# Copyright (c) 2014 Denis Laxalde <denis.laxalde@logilab.fr>
+# Copyright (c) 2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
+# Copyright (c) 2014 Eevee (Alex Munroe) <amunroe@yelp.com>
 # Copyright (c) 2015 Florian Bruhin <me@the-compiler.org>
 # Copyright (c) 2015 Radosław Ganczarek <radoslaw@ganczarek.in>
+# Copyright (c) 2016 Derek Gustafson <degustaf@gmail.com>
 # Copyright (c) 2016 Jakub Wilk <jwilk@jwilk.net>
+# Copyright (c) 2016 Ceridwen <ceridwenv@gmail.com>
+# Copyright (c) 2018 Ville Skyttä <ville.skytta@iki.fi>
+# Copyright (c) 2018 Mario Corchero <mcorcherojim@bloomberg.net>
+# Copyright (c) 2018 Mario Corchero <mariocj89@gmail.com>
+# Copyright (c) 2018 Anthony Sottile <asottile@umich.edu>
+# Copyright (c) 2019 Hugo van Kemenade <hugovk@users.noreply.github.com>
+# Copyright (c) 2019 markmcclain <markmcclain@users.noreply.github.com>
+# Copyright (c) 2019 BasPH <BasPH@users.noreply.github.com>
 
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
 # For details: https://github.com/PyCQA/astroid/blob/master/COPYING.LESSER
@@ -23,24 +35,26 @@ import imp
 import os
 import platform
 import sys
-from distutils.sysconfig import get_python_lib # pylint: disable=import-error
+import itertools
+from distutils.sysconfig import get_python_lib  # pylint: disable=import-error
+
 # pylint: disable=import-error, no-name-in-module
 from distutils.errors import DistutilsPlatformError
+
 # distutils is replaced by virtualenv with a module that does
 # weird path manipulations in order to get to the
 # real distutils module.
-
-import six
+from typing import Optional, List
 
 from .interpreter._import import spec
 from .interpreter._import import util
 
-if sys.platform.startswith('win'):
-    PY_SOURCE_EXTS = ('py', 'pyw')
-    PY_COMPILED_EXTS = ('dll', 'pyd')
+if sys.platform.startswith("win"):
+    PY_SOURCE_EXTS = ("py", "pyw")
+    PY_COMPILED_EXTS = ("dll", "pyd")
 else:
-    PY_SOURCE_EXTS = ('py',)
-    PY_COMPILED_EXTS = ('so',)
+    PY_SOURCE_EXTS = ("py",)
+    PY_COMPILED_EXTS = ("so",)
 
 
 try:
@@ -49,41 +63,42 @@ try:
     # with the prefix from which the virtualenv was created. This throws
     # off the detection logic for standard library modules, thus the
     # workaround.
-    STD_LIB_DIRS = set([
+    STD_LIB_DIRS = {
         get_python_lib(standard_lib=True, prefix=sys.prefix),
         # Take care of installations where exec_prefix != prefix.
         get_python_lib(standard_lib=True, prefix=sys.exec_prefix),
-        get_python_lib(standard_lib=True)])
+        get_python_lib(standard_lib=True),
+    }
 # get_python_lib(standard_lib=1) is not available on pypy, set STD_LIB_DIR to
 # non-valid path, see https://bugs.pypy.org/issue1164
 except DistutilsPlatformError:
     STD_LIB_DIRS = set()
 
-if os.name == 'nt':
-    STD_LIB_DIRS.add(os.path.join(sys.prefix, 'dlls'))
+if os.name == "nt":
+    STD_LIB_DIRS.add(os.path.join(sys.prefix, "dlls"))
     try:
         # real_prefix is defined when running inside virtual environments,
         # created with the **virtualenv** library.
-        STD_LIB_DIRS.add(os.path.join(sys.real_prefix, 'dlls'))
+        STD_LIB_DIRS.add(os.path.join(sys.real_prefix, "dlls"))
     except AttributeError:
         # sys.base_exec_prefix is always defined, but in a virtual environment
         # created with the stdlib **venv** module, it points to the original
         # installation, if the virtual env is activated.
         try:
-            STD_LIB_DIRS.add(os.path.join(sys.base_exec_prefix, 'dlls'))
+            STD_LIB_DIRS.add(os.path.join(sys.base_exec_prefix, "dlls"))
         except AttributeError:
             pass
 
-if platform.python_implementation() == 'PyPy':
-    _root = os.path.join(sys.prefix, 'lib_pypy')
+if platform.python_implementation() == "PyPy":
+    _root = os.path.join(sys.prefix, "lib_pypy")
     STD_LIB_DIRS.add(_root)
     try:
         # real_prefix is defined when running inside virtualenv.
-        STD_LIB_DIRS.add(os.path.join(sys.real_prefix, 'lib_pypy'))
+        STD_LIB_DIRS.add(os.path.join(sys.real_prefix, "lib_pypy"))
     except AttributeError:
         pass
     del _root
-if os.name == 'posix':
+if os.name == "posix":
     # Need the real prefix is we're under a virtualenv, otherwise
     # the usual one will do.
     try:
@@ -92,11 +107,11 @@ if os.name == 'posix':
         prefix = sys.prefix
 
     def _posix_path(path):
-        base_python = 'python%d.%d' % sys.version_info[:2]
+        base_python = "python%d.%d" % sys.version_info[:2]
         return os.path.join(prefix, path, base_python)
 
-    STD_LIB_DIRS.add(_posix_path('lib'))
-    if sys.maxsize > 2**32:
+    STD_LIB_DIRS.add(_posix_path("lib"))
+    if sys.maxsize > 2 ** 32:
         # This tries to fix a problem with /usr/lib64 builds,
         # where systems are running both 32-bit and 64-bit code
         # on the same machine, which reflects into the places where
@@ -104,10 +119,10 @@ if os.name == 'posix':
         # here http://bugs.python.org/issue1294959.
         # An easy reproducing case would be
         # https://github.com/PyCQA/pylint/issues/712#issuecomment-163178753
-        STD_LIB_DIRS.add(_posix_path('lib64'))
+        STD_LIB_DIRS.add(_posix_path("lib64"))
 
-EXT_LIB_DIR = get_python_lib()
-IS_JYTHON = platform.python_implementation() == 'Jython'
+EXT_LIB_DIRS = {get_python_lib(), get_python_lib(True)}
+IS_JYTHON = platform.python_implementation() == "Jython"
 BUILTIN_MODULES = dict.fromkeys(sys.builtin_module_names, True)
 
 
@@ -116,6 +131,7 @@ class NoSourceFile(Exception):
     source file for a precompiled file
     """
 
+
 def _normalize_path(path):
     return os.path.normcase(os.path.abspath(path))
 
@@ -126,12 +142,7 @@ def _canonicalize_path(path):
 
 def _path_from_filename(filename, is_jython=IS_JYTHON):
     if not is_jython:
-        if sys.version_info > (3, 0):
-            return filename
-        else:
-            if filename.endswith(".pyc"):
-                return filename[:-1]
-            return filename
+        return filename
     head, has_pyclass, _ = filename.partition("$py.class")
     if has_pyclass:
         return head + ".py"
@@ -152,6 +163,7 @@ def _handle_blacklist(blacklist, dirnames, filenames):
 
 _NORM_PATH_CACHE = {}
 
+
 def _cache_normalize_path(path):
     """abspath with caching"""
     # _module_file calls abspath on every path in sys.path every time it's
@@ -160,11 +172,12 @@ def _cache_normalize_path(path):
     try:
         return _NORM_PATH_CACHE[path]
     except KeyError:
-        if not path: # don't cache result for ''
+        if not path:  # don't cache result for ''
             return _normalize_path(path)
         result = _NORM_PATH_CACHE[path] = _normalize_path(path)
         return result
 
+
 def load_module_from_name(dotted_name, path=None, use_sys=True):
     """Load a Python module from its name.
 
@@ -187,19 +200,18 @@ def load_module_from_name(dotted_name, path=None, use_sys=True):
     :rtype: module
     :return: the loaded module
     """
-    return load_module_from_modpath(dotted_name.split('.'), path, use_sys)
+    return load_module_from_modpath(dotted_name.split("."), path, use_sys)
 
 
-def load_module_from_modpath(parts, path=None, use_sys=1):
+def load_module_from_modpath(parts, path: Optional[List[str]] = None, use_sys=1):
     """Load a python module from its split name.
 
     :type parts: list(str) or tuple(str)
     :param parts:
       python name of a module or package split on '.'
 
-    :type path: list or None
     :param path:
-      optional list of path where the module or package should be
+      Optional list of path where the module or package should be
       searched (use sys.path if nothing or None is given)
 
     :type use_sys: bool
@@ -213,14 +225,14 @@ def load_module_from_modpath(parts, path=None, use_sys=1):
     """
     if use_sys:
         try:
-            return sys.modules['.'.join(parts)]
+            return sys.modules[".".join(parts)]
         except KeyError:
             pass
     modpath = []
     prevmodule = None
     for part in parts:
         modpath.append(part)
-        curname = '.'.join(modpath)
+        curname = ".".join(modpath)
         module = None
         if len(modpath) != len(parts):
             # even with use_sys=False, should try to get outer packages from sys.modules
@@ -236,25 +248,26 @@ def load_module_from_modpath(parts, path=None, use_sys=1):
                 mp_file.close()
         if prevmodule:
             setattr(prevmodule, part, module)
-        _file = getattr(module, '__file__', '')
+        _file = getattr(module, "__file__", "")
         prevmodule = module
         if not _file and util.is_namespace(curname):
             continue
         if not _file and len(modpath) != len(parts):
-            raise ImportError('no module in %s' % '.'.join(parts[len(modpath):]))
+            raise ImportError("no module in %s" % ".".join(parts[len(modpath) :]))
         path = [os.path.dirname(_file)]
     return module
 
 
-def load_module_from_file(filepath, path=None, use_sys=True, extrapath=None):
+def load_module_from_file(
+    filepath: str, path: Optional[List[str]] = None, use_sys=True
+):
     """Load a Python module from it's path.
 
     :type filepath: str
     :param filepath: path to the python module or package
 
-    :type path: list or None
-    :param path:
-      optional list of path where the module or package should be
+    :param Optional[List[str]] path:
+      Optional list of path where the module or package should be
       searched (use sys.path if nothing or None is given)
 
     :type use_sys: bool
@@ -262,13 +275,12 @@ def load_module_from_file(filepath, path=None, use_sys=True, extrapath=None):
       boolean indicating whether the sys.modules dictionary should be
       used or not
 
-
     :raise ImportError: if the module or package is not found
 
     :rtype: module
     :return: the loaded module
     """
-    modpath = modpath_from_file(filepath, extrapath)
+    modpath = modpath_from_file(filepath)
     return load_module_from_modpath(modpath, path, use_sys)
 
 
@@ -279,51 +291,76 @@ def check_modpath_has_init(path, mod_path):
         modpath.append(part)
         path = os.path.join(path, part)
         if not _has_init(path):
-            old_namespace = util.is_namespace('.'.join(modpath))
+            old_namespace = util.is_namespace(".".join(modpath))
             if not old_namespace:
                 return False
     return True
 
 
-def modpath_from_file_with_callback(filename, extrapath=None, is_package_cb=None):
-    filename = _path_from_filename(filename)
-    filename = os.path.realpath(os.path.expanduser(filename))
-    base = os.path.splitext(filename)[0]
+def _get_relative_base_path(filename, path_to_check):
+    """Extracts the relative mod path of the file to import from
 
-    if extrapath is not None:
-        for path_ in six.moves.map(_canonicalize_path, extrapath):
-            path = os.path.abspath(path_)
-            if path and os.path.normcase(base[:len(path)]) == os.path.normcase(path):
-                submodpath = [pkg for pkg in base[len(path):].split(os.sep)
-                              if pkg]
-                if is_package_cb(path, submodpath[:-1]):
-                    return extrapath[path_].split('.') + submodpath
+    Check if a file is within the passed in path and if so, returns the
+    relative mod path from the one passed in.
 
-    for path in six.moves.map(_canonicalize_path, sys.path):
-        path = _cache_normalize_path(path)
-        if path and os.path.normcase(base).startswith(path):
-            modpath = [pkg for pkg in base[len(path):].split(os.sep) if pkg]
-            if is_package_cb(path, modpath[:-1]):
-                return modpath
+    If the filename is no in path_to_check, returns None
 
-    raise ImportError('Unable to find module for %s in %s' % (
-        filename, ', \n'.join(sys.path)))
+    Note this function will look for both abs and realpath of the file,
+    this allows to find the relative base path even if the file is a
+    symlink of a file in the passed in path
+
+    Examples:
+        _get_relative_base_path("/a/b/c/d.py", "/a/b") ->  ["c","d"]
+        _get_relative_base_path("/a/b/c/d.py", "/dev") ->  None
+    """
+    importable_path = None
+    path_to_check = os.path.normcase(path_to_check)
+    abs_filename = os.path.abspath(filename)
+    if os.path.normcase(abs_filename).startswith(path_to_check):
+        importable_path = abs_filename
+
+    real_filename = os.path.realpath(filename)
+    if os.path.normcase(real_filename).startswith(path_to_check):
+        importable_path = real_filename
+
+    if importable_path:
+        base_path = os.path.splitext(importable_path)[0]
+        relative_base_path = base_path[len(path_to_check) :]
+        return [pkg for pkg in relative_base_path.split(os.sep) if pkg]
+
+    return None
 
 
+def modpath_from_file_with_callback(filename, path=None, is_package_cb=None):
+    filename = os.path.expanduser(_path_from_filename(filename))
+    for pathname in itertools.chain(
+        path or [], map(_canonicalize_path, sys.path), sys.path
+    ):
+        pathname = _cache_normalize_path(pathname)
+        if not pathname:
+            continue
+        modpath = _get_relative_base_path(filename, pathname)
+        if not modpath:
+            continue
+        if is_package_cb(pathname, modpath[:-1]):
+            return modpath
 
-def modpath_from_file(filename, extrapath=None):
-    """given a file path return the corresponding split module's name
-    (i.e name of a module or package split on '.')
+    raise ImportError(
+        "Unable to find module for %s in %s" % (filename, ", \n".join(sys.path))
+    )
+
+
+def modpath_from_file(filename, path=None):
+    """Get the corresponding split module's name from a filename
+
+    This function will return the name of a module or package split on `.`.
 
     :type filename: str
     :param filename: file's path for which we want the module's name
 
-    :type extrapath: dict
-    :param extrapath:
-      optional extra search path, with path as key and package name for the path
-      as value. This is usually useful to handle package split in multiple
-      directories using __path__ trick.
-
+    :type Optional[List[str]] path:
+      Optional list of path where the module or package should be
+      searched (use sys.path if nothing or None is given)
 
     :raise ImportError:
       if the corresponding module's name has not been found
@@ -331,12 +368,13 @@ def modpath_from_file(filename, extrapath=None):
     :rtype: list(str)
     :return: the corresponding split module's name
     """
-    return modpath_from_file_with_callback(filename, extrapath, check_modpath_has_init)
+    return modpath_from_file_with_callback(filename, path, check_modpath_has_init)
 
 
 def file_from_modpath(modpath, path=None, context_file=None):
     return file_info_from_modpath(modpath, path, context_file).location
 
+
 def file_info_from_modpath(modpath, path=None, context_file=None):
     """given a mod path (i.e. split module / package name), return the
     corresponding file, giving priority to source file over precompiled
@@ -371,15 +409,17 @@ def file_info_from_modpath(modpath, path=None, context_file=None):
         context = os.path.dirname(context_file)
     else:
         context = context_file
-    if modpath[0] == 'xml':
+    if modpath[0] == "xml":
         # handle _xmlplus
         try:
-            return _spec_from_modpath(['_xmlplus'] + modpath[1:], path, context)
+            return _spec_from_modpath(["_xmlplus"] + modpath[1:], path, context)
         except ImportError:
             return _spec_from_modpath(modpath, path, context)
-    elif modpath == ['os', 'path']:
+    elif modpath == ["os", "path"]:
         # FIXME: currently ignoring search_path...
-        return spec.ModuleSpec(name='os.path', location=os.path.__file__, module_type=imp.PY_SOURCE)
+        return spec.ModuleSpec(
+            name="os.path", location=os.path.__file__, module_type=imp.PY_SOURCE
+        )
     return _spec_from_modpath(modpath, path, context)
 
 
@@ -410,9 +450,9 @@ def get_module_part(dotted_name, context_file=None):
     (see #10066)
     """
     # os.path trick
-    if dotted_name.startswith('os.path'):
-        return 'os.path'
-    parts = dotted_name.split('.')
+    if dotted_name.startswith("os.path"):
+        return "os.path"
+    parts = dotted_name.split(".")
     if context_file is not None:
         # first check for builtin module which won't be considered latter
         # in that case (path != None)
@@ -423,22 +463,24 @@ def get_module_part(dotted_name, context_file=None):
         # don't use += or insert, we want a new list to be created !
     path = None
     starti = 0
-    if parts[0] == '':
-        assert context_file is not None, \
-                'explicit relative import, but no context_file?'
-        path = [] # prevent resolving the import non-relatively
+    if parts[0] == "":
+        assert (
+            context_file is not None
+        ), "explicit relative import, but no context_file?"
+        path = []  # prevent resolving the import non-relatively
         starti = 1
-    while parts[starti] == '': # for all further dots: change context
+    while parts[starti] == "":  # for all further dots: change context
         starti += 1
         context_file = os.path.dirname(context_file)
     for i in range(starti, len(parts)):
         try:
-            file_from_modpath(parts[starti:i+1], path=path,
-                              context_file=context_file)
+            file_from_modpath(
+                parts[starti : i + 1], path=path, context_file=context_file
+            )
         except ImportError:
             if i < max(1, len(parts) - 2):
                 raise
-            return '.'.join(parts[:i])
+            return ".".join(parts[:i])
     return dotted_name
 
 
@@ -465,9 +507,11 @@ def get_module_files(src_directory, blacklist, list_all=False):
     """
     files = []
     for directory, dirnames, filenames in os.walk(src_directory):
+        if directory in blacklist:
+            continue
         _handle_blacklist(blacklist, dirnames, filenames)
         # check for __init__.py
-        if not list_all and '__init__.py' not in filenames:
+        if not list_all and "__init__.py" not in filenames:
             dirnames[:] = ()
             continue
         for filename in filenames:
@@ -479,7 +523,7 @@ def get_module_files(src_directory, blacklist, list_all=False):
 
 def get_source_file(filename, include_no_ext=False):
     """given a python module's file name return the matching source file
-    name (the filename will be returned identically if it's a already an
+    name (the filename will be returned identically if it's already an
     absolute path to a python source file...)
 
     :type filename: str
@@ -494,7 +538,7 @@ def get_source_file(filename, include_no_ext=False):
     filename = os.path.abspath(_path_from_filename(filename))
     base, orig_ext = os.path.splitext(filename)
     for ext in PY_SOURCE_EXTS:
-        source_path = '%s.%s' % (base, ext)
+        source_path = "%s.%s" % (base, ext)
         if os.path.exists(source_path):
             return source_path
     if include_no_ext and not orig_ext and os.path.exists(base):
@@ -527,7 +571,7 @@ def is_standard_module(modname, std_path=None):
       - is located on the path listed in one of the directory in `std_path`
       - is a built-in module
     """
-    modname = modname.split('.')[0]
+    modname = modname.split(".")[0]
     try:
         filename = file_from_modpath([modname])
     except ImportError:
@@ -540,8 +584,9 @@ def is_standard_module(modname, std_path=None):
         # we assume there are no namespaces in stdlib
         return not util.is_namespace(modname)
     filename = _normalize_path(filename)
-    if filename.startswith(_cache_normalize_path(EXT_LIB_DIR)):
-        return False
+    for path in EXT_LIB_DIRS:
+        if filename.startswith(_cache_normalize_path(path)):
+            return False
     if std_path is None:
         std_path = STD_LIB_DIRS
     for path in std_path:
@@ -550,7 +595,6 @@ def is_standard_module(modname, std_path=None):
     return False
 
 
-
 def is_relative(modname, from_file):
     """return true if the given module name is relative to the given
     file name
@@ -571,7 +615,7 @@ def is_relative(modname, from_file):
     if from_file in sys.path:
         return False
     try:
-        stream, _, _ = imp.find_module(modname.split('.')[0], [from_file])
+        stream, _, _ = imp.find_module(modname.split(".")[0], [from_file])
 
         # Close the stream to avoid ResourceWarnings.
         if stream:
@@ -583,6 +627,7 @@ def is_relative(modname, from_file):
 
 # internal only functions #####################################################
 
+
 def _spec_from_modpath(modpath, path=None, context=None):
     """given a mod path (i.e. split module / package name), return the
     corresponding spec
@@ -604,7 +649,9 @@ def _spec_from_modpath(modpath, path=None, context=None):
     if found_spec.type == spec.ModuleType.PY_COMPILED:
         try:
             location = get_source_file(found_spec.location)
-            return found_spec._replace(location=location, type=spec.ModuleType.PY_SOURCE)
+            return found_spec._replace(
+                location=location, type=spec.ModuleType.PY_SOURCE
+            )
         except NoSourceFile:
             return found_spec._replace(location=location)
     elif found_spec.type == spec.ModuleType.C_BUILTIN:
@@ -621,25 +668,23 @@ def _is_python_file(filename):
 
     .pyc and .pyo are ignored
     """
-    for ext in ('.py', '.so', '.pyd', '.pyw'):
-        if filename.endswith(ext):
-            return True
-    return False
+    return filename.endswith((".py", ".so", ".pyd", ".pyw"))
 
 
 def _has_init(directory):
     """if the given directory has a valid __init__ file, return its path,
     else return None
     """
-    mod_or_pack = os.path.join(directory, '__init__')
-    for ext in PY_SOURCE_EXTS + ('pyc', 'pyo'):
-        if os.path.exists(mod_or_pack + '.' + ext):
-            return mod_or_pack + '.' + ext
+    mod_or_pack = os.path.join(directory, "__init__")
+    for ext in PY_SOURCE_EXTS + ("pyc", "pyo"):
+        if os.path.exists(mod_or_pack + "." + ext):
+            return mod_or_pack + "." + ext
     return None
 
 
 def is_namespace(specobj):
     return specobj.type == spec.ModuleType.PY_NAMESPACE
 
+
 def is_directory(specobj):
     return specobj.type == spec.ModuleType.PKG_DIRECTORY
